package com.nakkaya.gui;

import javax.swing.JFrame;
import javax.swing.JTabbedPane;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.JScrollPane;
import java.awt.Dimension;
import javax.swing.ListSelectionModel;
import java.net.URL;
import java.awt.Image;
import java.awt.Toolkit;
import javax.swing.Box;
import javax.swing.JLabel;
import javax.swing.JTextField;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import javax.swing.JList;
import java.util.logging.Logger;
import java.util.logging.Level;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.prefs.Preferences;
import javax.swing.JCheckBox;
import java.awt.BorderLayout;
import java.awt.event.KeyEvent;
import javax.swing.KeyStroke;
import javax.swing.Action;
import java.awt.event.ActionEvent;
import javax.swing.JComponent;
import javax.swing.AbstractAction;
import javax.swing.JButton;
import java.awt.Font;
import com.nakkaya.arp.ArpWatcher;
import com.nakkaya.arp.ArpTable;
import com.nakkaya.arp.ArpTableModel;
import com.nakkaya.network.Network;
import javax.swing.Timer;

public class InfoWindow extends JFrame {
    public static final boolean NOTIFY_FIREWALL_ACTIVITY = true;
    public static final boolean SUPPRESS_INCOMPLETE_WARNING = true;

    Preferences preferences = Preferences.userRoot();

    SliderLogHandler sliderLogHandler = new SliderLogHandler();

    Logger arpLogger = Logger.getLogger("ArpLog");
    Logger firewallLogger = Logger.getLogger("FirewallLog");

    JTabbedPane tabs = new JTabbedPane();

    JPanel networkPanel = new JPanel();
    JTable arpTable = new JTable();
    ArpTableModel arpTableModel = null;
    JPanel arpPanel = new JPanel(new BorderLayout());
    JPanel arpLogPanel = new JPanel(new BorderLayout());
    JPanel firewallLogPanel = new JPanel(new BorderLayout());
    JPanel optionsPanel = new JPanel();

    Network network = new Network();
    JLabel adapterLabel;
    JLabel IPLabel;
    JLabel wanIPLabel;
    JLabel routerIPLabel;

    public InfoWindow( ArpTableModel mdl ){
	arpTableModel = mdl;	

	setTitle("Mocha");
	URL imageURL = this.getClass().getClassLoader()
	    .getResource( "icons/mocha25.png" );
	Image image = 
	    Toolkit.getDefaultToolkit().getImage(imageURL);
	setIconImage( image );

	//setup arp table
	//set table
        arpTable.setIntercellSpacing(new Dimension());
	// Allow only single a selection
	arpTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	arpTable.setModel( arpTableModel );
	
	//arpTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
	arpTable.getColumnModel().getColumn(0).setPreferredWidth(160);
	arpTable.getColumnModel().getColumn(1).setPreferredWidth(140);
	arpTable.getColumnModel().getColumn(2).setPreferredWidth(195);

	JScrollPane arpTableScrollPane = new JScrollPane();
	arpTableScrollPane.setViewportView(arpTable);
	arpPanel.add( arpTableScrollPane );

	//setup arp log panel
	LogListModel logList = new LogListModel();
	ListLogHandler arpLogHandler = new ListLogHandler( logList );
	arpLogger.addHandler( arpLogHandler );
	arpLogger.addHandler( sliderLogHandler );	

	JList arpLog = new JList( logList );
	JScrollPane arpLogscrollPane = new JScrollPane();
	arpLogscrollPane.setViewportView(arpLog);
	arpLogPanel.add( arpLogscrollPane );

	LogListModel firewallLog = new LogListModel();
	ListLogHandler firewallLogHandler = new ListLogHandler( firewallLog );
	firewallLogger.addHandler( firewallLogHandler );
	if ( preferences.getBoolean("watchFirewall" , NOTIFY_FIREWALL_ACTIVITY)== true)
	    firewallLogger.addHandler( sliderLogHandler );;

	JList firewallLogList = new JList( firewallLog );
	JScrollPane firewallLogScrollPane = new JScrollPane( firewallLogList );
	firewallLogScrollPane.setViewportView ( firewallLogList );
	firewallLogPanel.add( firewallLogScrollPane );
	
	createNetworkPanel();
	createOptionsPanel();
	
	//set all dimensions here...
	Dimension dim = new Dimension( 500 , 300 );
	arpTableScrollPane.setPreferredSize( dim );
	arpLogscrollPane.setPreferredSize( dim );
	firewallLogScrollPane.setPreferredSize ( dim );
	
	tabs.add( networkPanel , "Network" );
	tabs.add( arpPanel , "Arp Table" );
	tabs.add( arpLogPanel , "Arp Log" );
	tabs.add( firewallLogPanel , "Firewall Log" );
	tabs.add( optionsPanel , "Options" );

	initKeyBindings();

	add(tabs);
	pack();
    }

    public void createOptionsPanel(){
	//setup options
	optionsPanel.setLayout(new GridBagLayout() );
	GridBagConstraints c = new GridBagConstraints();
	c.anchor= GridBagConstraints.WEST;

	c.gridx = 0;
	c.gridy = 0;
	c.insets = new Insets(10,0,0,20);
 	JLabel downloadLabel = new JLabel("Arp Table Refresh Interval ");
	downloadLabel.setFont(new Font("Arial",0,14));
 	optionsPanel.add(downloadLabel , c );

	c.gridx = 1;
	c.gridy = 0;
	Box box = Box.createHorizontalBox();
	JTextField watchArpValue = new JTextField( 3 );	
	watchArpValue.setFont(new Font("Arial",0,14));
	
	Integer tmpValue = preferences.getInt( "arpWatchInterval", ArpWatcher.SLEEP );
 	watchArpValue.setText( tmpValue.toString() );

 	box.add( watchArpValue );
	box.add( Box.createRigidArea( new Dimension( 10 , 0 )));
	JLabel label = new JLabel(" seconds");
	label.setFont(new Font("Arial",0,14));
 	box.add( label );
	optionsPanel.add( box , c );

	ActionListener watchArpListener = new ActionListener() { 
		public void actionPerformed(ActionEvent e) {
		    JTextField field = (JTextField)e.getSource();		    
		    int arpWatchInterval = 5;
		    try{
			arpWatchInterval = Integer.parseInt( field.getText() );
		    }catch( Exception ex ) { 
			arpWatchInterval = 5;
		    }
		    preferences.putInt( "arpWatchInterval", arpWatchInterval );
		}};
	watchArpValue.addActionListener( watchArpListener );

	c.gridx = 0;
	c.gridy = 1;
	c.insets = new Insets(10,0,0,20);
 	JLabel notifyFirewall = new JLabel("Notify Firewall Activity:");
	notifyFirewall.setFont(new Font("Arial",0,14));
 	optionsPanel.add(notifyFirewall , c );
	
	c.gridx = 1;
	c.gridy = 1;
	c.insets = new Insets(10,0,0,20);
	JCheckBox notifyFirewallCheckBox = new JCheckBox();
	notifyFirewallCheckBox.setFont(new Font("Arial",0,14));
	notifyFirewallCheckBox.setSelected
	    (preferences.getBoolean("watchFirewall" , NOTIFY_FIREWALL_ACTIVITY));
	optionsPanel.add( notifyFirewallCheckBox , c );


	ActionListener watchFirewallListener = new ActionListener() { 
		public void actionPerformed(ActionEvent e) {
		    JCheckBox field = (JCheckBox)e.getSource();		    
		    boolean watchFirewall = field.isSelected();
		    
		    LogListModel firewallLog = new LogListModel();
		    ListLogHandler firewallLogHandler = 
			new ListLogHandler( firewallLog );

		    if ( watchFirewall == true)
			firewallLogger.addHandler( sliderLogHandler );
		    else
			firewallLogger.removeHandler( sliderLogHandler );

		    preferences.putBoolean( "watchFirewall", watchFirewall );
		}};
	notifyFirewallCheckBox.addActionListener( watchFirewallListener );

	c.gridx = 0;
	c.gridy = 2;
	c.insets = new Insets(10,0,0,20);
	label = new JLabel("Show Last ");
	label.setFont(new Font("Arial",0,14));
 	optionsPanel.add( label , c );

	c.gridx = 1;
	c.gridy = 2;
	c.insets = new Insets(10,0,0,20);

	box = Box.createHorizontalBox();
	JTextField showLastMessages = new JTextField( 3 );
	showLastMessages.setFont(new Font("Arial",0,14));
	tmpValue = preferences.getInt
	    ( "maxLogListSize", LogListModel.MAX_SIZE );
	showLastMessages.setText( tmpValue.toString() );
	box.add( showLastMessages );
	box.add( Box.createRigidArea( new Dimension( 10 , 0 )));
	label = new JLabel(" log messages.");
	label.setFont(new Font("Arial",0,14));
 	box.add( label );

	showLastMessages.addActionListener( new ActionListener() { 
		public void actionPerformed(ActionEvent e) {
		    JTextField field = (JTextField)e.getSource();		    
		    int maxLogs = LogListModel.MAX_SIZE;
		    try{
			maxLogs = Integer.parseInt( field.getText() );
		    }catch( Exception ex ) { 
			maxLogs = 5;
		    }
		    preferences.putInt( "maxLogListSize", maxLogs );
		}});

	optionsPanel.add( box , c );


	c.gridx = 0;
	c.gridy = 3;
	c.insets = new Insets(10,0,0,20);
 	label = new JLabel("Suppress Incomplete Warnings:");
	label.setFont(new Font("Arial",0,14));
 	optionsPanel.add(label , c );
	
	c.gridx = 1;
	c.gridy = 3;
	c.insets = new Insets(10,0,0,20);
	JCheckBox suppressIncompleteCheckBox = new JCheckBox();
	suppressIncompleteCheckBox.setFont(new Font("Arial",0,14));
	suppressIncompleteCheckBox.setSelected
	    (preferences.getBoolean("suppressIncompleteWarning" ,
				    SUPPRESS_INCOMPLETE_WARNING));
	optionsPanel.add( suppressIncompleteCheckBox , c );


	ActionListener watchSuppressIncompleteListener = new ActionListener() { 
		public void actionPerformed(ActionEvent e) {
		    JCheckBox field = (JCheckBox)e.getSource();		    
		    boolean suppress = field.isSelected();		    

		    preferences.putBoolean( "suppressIncompleteWarning",suppress );
		}};
	suppressIncompleteCheckBox.addActionListener(watchSuppressIncompleteListener);

	c.gridx = 0;
	c.gridy = 4;
	c.insets = new Insets(10,0,0,20);
 	label = new JLabel("Arp Command:");
	label.setFont(new Font("Arial",0,14));
 	optionsPanel.add(label , c );	

	c.gridx = 1;
	c.gridy = 4;
	c.insets = new Insets(10,0,0,20);
	JTextField command = new JTextField(15);
	command.setFont(new Font("Arial",0,14));
	command.setText( preferences.get( "arpCommand", ArpWatcher.ARP_COMMAND ) );
 	optionsPanel.add(command , c );

	command.addActionListener( new ActionListener() { 
		public void actionPerformed(ActionEvent e) {
		    JTextField field = (JTextField)e.getSource();
		    preferences.put( "arpCommand", field.getText() );
		}});

	c.gridx = 0;
	c.gridy = 5;
	c.insets = new Insets(10,0,0,20);
	optionsPanel.add( Box.createVerticalStrut( 100 ) , c );

    }

    public void createNetworkPanel(){
	networkPanel.setLayout(new GridBagLayout() );
	GridBagConstraints c = new GridBagConstraints();
	c.anchor= GridBagConstraints.WEST;

	adapterLabel = new JLabel( network.adapter );
	IPLabel = new JLabel( network.localIP );
	wanIPLabel = new JLabel( network.wanIP );
	routerIPLabel = new JLabel( network.gateway );


	c.gridx = 0;
	c.gridy = 0;
	c.insets = new Insets(20,0,0,20);
	JLabel label = new JLabel("Default Adapter: ");
	label.setFont(new Font("Arial",0,18));
 	networkPanel.add( label  , c );

	c.gridx = 1;
	c.gridy = 0;
	c.insets = new Insets(20,0,0,20);
	adapterLabel.setFont(new Font("Arial",0,18));
 	networkPanel.add( adapterLabel , c );

	c.gridx = 0;
	c.gridy = 1;
	c.insets = new Insets(20,0,0,20);
	label = new JLabel("IP Address: ");
	label.setFont(new Font("Arial",0,18));
 	networkPanel.add( label , c );

	c.gridx = 1;
	c.gridy = 1;
	c.insets = new Insets(20,0,0,20);
	IPLabel.setFont(new Font("Arial",0,18));
 	networkPanel.add( IPLabel , c );

	c.gridx = 0;
	c.gridy = 2;
	c.insets = new Insets(20,0,0,20);
	label = new JLabel("WAN IP Address: ");
	label.setFont(new Font("Arial",0,18));
 	networkPanel.add( label , c );

	c.gridx = 1;
	c.gridy = 2;
	c.insets = new Insets(20,0,0,20);
	wanIPLabel.setFont(new Font("Arial",0,18));
 	networkPanel.add( wanIPLabel , c );

	c.gridx = 0;
	c.gridy = 3;
	c.insets = new Insets(20,0,0,20);
	label = new JLabel("Router IP: ");
	label.setFont(new Font("Arial",0,18));
 	networkPanel.add( label , c );

	c.gridx = 1;
	c.gridy = 3;
	c.insets = new Insets(20,0,0,20);
	routerIPLabel.setFont(new Font("Arial",0,18));
 	networkPanel.add( routerIPLabel , c );

	ActionListener updateNetworkPanel = new ActionListener() { 
		public void actionPerformed(ActionEvent e) { 
		    adapterLabel.setText( network.adapter );
		    IPLabel.setText( network.localIP );
		    wanIPLabel.setText( network.wanIP );
		    routerIPLabel.setText( network.gateway );
		}};
	Timer updateNetworkTimer = new Timer ( 5000 , updateNetworkPanel); 
	updateNetworkTimer.start();

	c.gridx = 0;
	c.gridy = 4;
	c.insets = new Insets(0,0,0,20);
	networkPanel.add( Box.createVerticalStrut( 40 ) , c );

    }

    public void initKeyBindings(){
// 	tabs.addKeyListener(new java.awt.event.KeyAdapter() {
// 		public void keyPressed(java.awt.event.KeyEvent evt) {
// 		    //formKeyPressed(evt);
// 		    System.out.println( "Pressed - " + evt.getKeyChar() ) ;
// 		}
// 		public void keyTyped(java.awt.event.KeyEvent evt) {
// 		    System.out.println( "Typed " + evt.getKeyChar() ) ;
// 		}
// 	    });

	//os specific modifier key
	int modifierKey = java.awt.event.InputEvent.ALT_MASK;
	if ( preferences.get("OS" , Network.CURRENT_OS ).equals( "OSX" ) ){
	    modifierKey = java.awt.event.InputEvent.META_MASK;
	}

	//bind command + w
	KeyStroke altF = KeyStroke.getKeyStroke
	    (KeyEvent.VK_W,modifierKey,false);

	Action alt_F_Action = new AbstractAction(){
		public void actionPerformed(ActionEvent e){
		    setVisible( false );
		}
	    };
	tabs.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(altF, "ALT_F");
	tabs.getActionMap().put("ALT_F", alt_F_Action);

	//bind tab movement
	KeyStroke altRight = KeyStroke.getKeyStroke
	    (KeyEvent.VK_RIGHT,modifierKey,false);

	Action alt_Right_Action = new AbstractAction(){
		public void actionPerformed(ActionEvent e){
		    int total = tabs.getTabCount();
		    int index = tabs.getSelectedIndex();
		    
		    if ( index+1 >= total )
			index = 0;
		    else
			index++;

		    tabs.setSelectedIndex( index );
		    tabs.requestFocusInWindow();
		}
	    };
	tabs.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(altRight, "ALT_RIGHT");
	tabs.getActionMap().put("ALT_RIGHT", alt_Right_Action);

	KeyStroke altLeft = KeyStroke.getKeyStroke
	    (KeyEvent.VK_LEFT,modifierKey,false);

	Action alt_Left_Action = new AbstractAction(){
		public void actionPerformed(ActionEvent e){
		    int total = tabs.getTabCount();
		    int index = tabs.getSelectedIndex();

		    if ( index == 0 )
			index = total-1;
		    else
			index--;

		    tabs.setSelectedIndex( index );
		    tabs.requestFocusInWindow();
		}
	    };

	tabs.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(altLeft, "ALT_LEFT");
	tabs.getActionMap().put("ALT_LEFT", alt_Left_Action);
    }
}
