import java.lang.String;
import java.util.logging.Logger;
import java.util.Vector;
import javax.swing.JFrame;
import java.lang.reflect.Constructor;

import java.util.logging.ConsoleHandler;
import java.util.logging.Formatter;
import java.util.logging.Handler;
import java.util.logging.LogRecord;
import java.util.logging.Logger;
import java.util.logging.Level;
import java.util.logging.FileHandler;
import javax.swing.LookAndFeel;
import javax.swing.UIManager;
import java.util.prefs.Preferences;

import com.nakkaya.arp.*;
import com.nakkaya.tail.*;
import com.nakkaya.gui.*;
import com.nakkaya.network.*;

class mocha {

    static Logger logger = Logger.getLogger("ArpLog");
    static Logger firewallLogger = Logger.getLogger("FirewallLog");
    static Logger sysLogger = Logger.getLogger("SysLog");
    static Preferences preferences = Preferences.userRoot();

    public static void detectOS(){
	if(System.getProperty("mrj.version") != null){

	    String os = "OSX";

	    String appDataLocation = 
		System.getProperty( "user.home" ) 
		+ "/Library/Application Support/Mocha/";

	    preferences.put( "OS", os );
	    preferences.put( "appDataLocation", appDataLocation );

	    System.setProperty("apple.awt.brushMetalLook", "true" );

	}else if (System.getProperty("os.name").equals("Linux") == true){
	    String os = "Linux";
	    String appDataLocation = 
		System.getProperty( "user.home" ) + "/.mocha/";

	    preferences.put( "OS", os );
	    preferences.put( "appDataLocation", appDataLocation );

	    //look and feel
	    try{
		UIManager.setLookAndFeel
		    ("com.sun.java.swing.plaf.gtk.GTKLookAndFeel");
	    }catch( Exception e ) { 
		sysLogger.warning(e.toString());
	    }
	}
    }

    public static void main(String[] argsCommand) {
	try{
	    logger.setLevel(Level.ALL);
	    logger.setUseParentHandlers(false);

	    firewallLogger.setLevel(Level.ALL);
	    firewallLogger.setUseParentHandlers(false);

	    detectOS();

	    ArpTable arpTable = new ArpTable();
	    ArpTableModel arpTableModel = new ArpTableModel( arpTable );

	    InfoWindow infoWindow = new InfoWindow( arpTableModel );
 	    infoWindow.setLocationRelativeTo( null );
 	    infoWindow.setVisible( true );

	    //start arp watcher thread
	    ArpWatcher arpWatcher = new ArpWatcher( arpTable , arpTableModel );
	    Thread thread = new Thread( arpWatcher , "ArpWathcer");
	    thread.setName("ArpWatcher");
	    thread.start();

	    //start log watcher
	    try{		
		if (preferences.get("OS" , Network.CURRENT_OS ).equals( "OSX" ) == true ){
		    Tail tailFirewall = new Tail("/var/log/appfirewall.log");
		}
		if (preferences.get("OS" , Network.CURRENT_OS).equals( "Linux" ) == true ){
		    Tail tailFirewall = new Tail("/var/log/messages");
		    tailFirewall.grep("IPTABLES: ");
		}
	    }catch( Exception e ) { 
		sysLogger.warning( e.toString() );
		firewallLogger.warning( "Read Permissin Required.");
	    }

	    //load OS specific features.
	    if (preferences.get("OS" , Network.CURRENT_OS ).equals( "OSX" ) == true ){
		try{
		    Class klass = Class.forName("com.nakkaya.mac.MacApplication");
		    Class[] paramTypes = { JFrame.class };
		    Constructor cons = klass.getConstructor(paramTypes);
		    
		    Object[] args = { infoWindow };		
		    Object theObject = cons.newInstance(args);
		    infoWindow.setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE);
		}catch( Exception e ) {
		    logger.warning( e.toString() );
		}
	    }

	    if (preferences.get("OS" , Network.CURRENT_OS).equals( "Linux" ) == true ){
		try{
		    //set to exit on unavaible tray
		    infoWindow.setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE);
		    Class klass = Class.forName("com.nakkaya.tray.SysTray");
		    Class[] paramTypes = { InfoWindow.class };
		    Constructor cons = klass.getConstructor(paramTypes);
		    
		    Object[] args = { infoWindow };		
		    Object theObject = cons.newInstance(args);
		}catch( Exception e ) {
		    logger.warning( e.toString() );
		}
	    }

	}catch( Exception e ) { 
	    logger.warning( e.toString() );
	}
    }
}
